<form action="{{action}}" method="post">
    %
    <input type="text" name="login" id="login" value="{{user_login}}" />
    %
    <input type="password" name="password" id="password" />
    <button type="submit" name="login_submit">Submit</button>
</form>
